/*
    PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
    Copyright 2002 Netscape Communications Corp. All rights reserved.
*/


/* ----------------------------- gzip-search ----------------------------- */

/* findCompressed.c:  If the requested file is there, check to see if it
 *                    has a compressed counterpart
 */

/*   Joe McCabe - 2002
 *
 *   Version history:
 *
 *   May 14, 2002 - v.1 This does a real brute force check. First it
 *   does a stat() to make sure the uncompressed file exists (if so
 *   configured - default behavior is to not do an initial stat()),
 *   then looks for a compressed form of the same file in the same
 *   location.
 */

/* This function will look at the given file name/path. The function
   will look for a file with the same base name/path, but with the
   added extension .gz. If found, the function will use the new path
   and convince the rest of the server to act accordingly. Note that
   this requires an extra stat (or two, depending on configuration)
   call for every HTML file accessed.

   Usage:
   At the end of magnus.conf:
      Init fn=load-modules shlib=<path to>findCompressed.<ext> funcs=findCompressed

   <ext> = so on UNIX
   <ext> = dll on NT

   Inside an object in obj.conf:
      PathCheck fn="findCompressed" failover="[true|false|0|1|yes|no]"

   The PathCheck directive must appear after any other PathChecks in
   the obj.conf

   NOTE:
   This is a proof of concept and has room for improvement. This
   solution breaks in several ways:

   1) Browser support for compression
      It is very possible that a browser will not understand how to
      work with compressed data. This NSAPI does not make any checks
      to see if the browser supports compressed data. NES6 (and
      greater) can be configured to check for compression support (see
      below). Note also that some browsers prefer a Content-encoding:
      type of "x-gzip" while others will prefer "gzip." NES uses
      "x-gzip" by default (set in the mime.types file). The following
      configuration example will send compressed data if the browser
      claims support for it, and will set the Content-encoding: to
      match what the browser sent. The second <Client> tag should
      immediately preceed the send-file SAF.

          # Will execute the findCompressed function only if the
          # requested file has an extension of .htm or .html, and
          # the browser claims support for gzip data
      <Client path="*.(html|htm)" headers="accept-encoding=*gzip*">
      PathCheck fn=findCompressed
      </Client>
      ...

          # If the browser does not claim support for x-gzip but the
          # Content-encoding: header is set to "x-gzip" then change
          # the Content-encoding: header to "gzip". Note that if you
          # change "x-gzip" to "gzip" in the mime.types file then this
          # logic would need to be reversed. This should be placed
          # immediately before the send-file Service.
      <Client headers="accept-encoding=*~x-gzip*" srvhdrs="content-encoding=x-gzip"
      Service fn="set-variable" set-srvhdrs="content-encoding=gzip"
      </Client>

   2) Realtime compression of dynamic content
      NES6x does not support post-processing of content. This means
      that NES can not compress content generated by subsystems (Java,
      SHTML, CGI, NSAPI, etc). Compression of data from these systems
      would have to be written into the system (i.e. a Java filter to
      provide compression of data coming from servlets/JSPs, zlib
      inclusion in CGI/NSAPI, etc).

   3) Insertion of data into static data
      Dynamic headers/footers, and other SAFs that would insert data
      into a static document are not able to unpack, modify, and
      repack the compressed form of data.
 */

#ifdef XP_WIN32
#define NSAPI_PUBLIC __declspec(dllexport)
#else  /* !XP_WIN32 */
#define NSAPI_PUBLIC
#endif /* !XP_WIN32 */

#include "nsapi.h"

#ifdef __cplusplus
extern "C"
#endif


NSAPI_PUBLIC int findCompressed(pblock *pb, Session *sn, Request *rq)
{
  char *path = pblock_findval("path", rq->vars);
  char *new_path;
  int return_value = REQ_NOACTION;
  int path_len;
  struct stat finfo;
  char *failover = pblock_findval("failover", pb);

/* ----------------------------------------------------------
 * Determine if we should bail out if an uncompressed copy of the file
 * does not exist. By default we will allow processing to continue. A
 * "failover" value of "false", "0", or "no" will make us bail.
 * ---------------------------------------------------------- */
  if ((failover != 0) &&
      ((PL_strcasecmp(failover, "no") == 0) ||
       (PL_strcasecmp(failover, "false") == 0) ||
       (strcmp(failover, "0") == 0)))
    {
      if(stat(path, &finfo) == -1) {
        goto done;
      }
    }

  /* Make room for ".gz", then make it */
  path_len = strlen(path);
  new_path = (char *)MALLOC(path_len + 3 + 1);
  strncpy(new_path, path, path_len);
  strcpy(&new_path[path_len], ".gz");

  /* Check for the compressed file - bail out on failure */
  if(stat(new_path, &finfo) == -1) {
    goto done;
  }

  /* Log a message explaining what we've done */
  log_ereport(LOG_VERBOSE, "Sent compressed file %s instead of %s", new_path, path);

  /* Update the path */
  param_free(pblock_remove("path", rq->vars));
  pblock_nvinsert("path", new_path, rq->vars);

  /* NES caches stat() - update */
  (void) request_stat_path(NULL, rq);

  /* We're all done */
  goto done;

 done:
  /* FREE our MALLOC */
  if (new_path) FREE(new_path);

  /* Return Value */
  return(return_value);
}
